from pathlib import Path
from tempfile import NamedTemporaryFile
from unittest.mock import patch
from zipfile import ZipFile

import pytest
from apistar.exceptions import ErrorResponse

from worker_file_import.transkribus import TranskribusElement, TranskribusFolder

SAMPLES = Path(__file__).absolute().parent / "samples"


def test_get_folder(mock_worker):
    mock_worker.api_client.add_response(
        "ListElements",
        corpus="corpusid",
        name="Untitled (3)",
        type="book",
        response=[
            {
                "id": "wrongfolderid",
                "name": "Untitled (3) (2)",
            },
            {
                "id": "folderid",
                "name": "Untitled (3)",
            },
        ],
    )

    folder = TranskribusFolder(
        mock_worker,
        # archive and filenames are not needed for this test, but required by the class
        archive=None,
        filenames=[],
        title="Untitled (3)",
        project_id="corpusid",
        folder_type="book",
        element_type="page",
        paragraph_type="paragraph",
        line_type="line",
    )

    folder.get_or_create_folder()

    assert folder.folder == {
        "id": "folderid",
        "name": "Untitled (3)",
    }


def test_create_folder(mock_worker):
    mock_worker.api_client.add_response(
        "ListElements",
        corpus="corpusid",
        name="Untitled (3)",
        type="book",
        response=[],
    )
    mock_worker.api_client.add_response(
        "CreateElement",
        body={
            "corpus": "corpusid",
            "type": "book",
            "name": "Untitled (3)",
            "worker_run_id": mock_worker.worker_run_id,
        },
        response={
            "id": "folderid",
            "type": "book",
            "name": "Untitled (3)",
            "corpus": {"id": "corpusid"},
        },
    )

    folder = TranskribusFolder(
        mock_worker,
        # archive and filenames are not needed for this test, but required by the class
        archive=None,
        filenames=[],
        title="Untitled (3)",
        project_id="corpusid",
        folder_type="book",
        element_type="page",
        paragraph_type="paragraph",
        line_type="line",
    )

    folder.get_or_create_folder()

    assert folder.folder == {
        "id": "folderid",
        "type": "book",
        "name": "Untitled (3)",
        "corpus": {"id": "corpusid"},
    }


@patch("worker_file_import.transkribus.hash_image")
def test_upload_image(hash_image_mock, mock_worker, responses):
    hash_image_mock.return_value = "01e9327d0e5ab7c516a0792c3662bc65"
    mock_worker.api_client.add_response(
        "CreateImage",
        body={"hash": "01e9327d0e5ab7c516a0792c3662bc65"},
        response={
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "s3_put_url": "http://s3/put",
            "s3_url": "http://s3/image.jpg",
        },
    )
    responses.add(responses.PUT, "http://s3/put", status=201)
    mock_worker.api_client.add_response(
        "PartialUpdateImage",
        id="aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
        body={"status": "checked"},
        response={
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "s3_put_url": None,
            "width": 999,
            "height": 999,
            "status": "checked",
        },
    )

    with (
        NamedTemporaryFile() as archive_file,
        ZipFile(archive_file, mode="w") as archive,
    ):
        archive.write(str(SAMPLES / "200x200.jpg"), arcname="0001_123456.jpg")
        archive.write(str(SAMPLES / "transcript.xml"), arcname="0001_123456.xml")
        archive_file.flush()
        archive_file.seek(0)

        test_element = TranskribusElement(
            mock_worker,
            archive,
            image_filename="0001_123456.jpg",
            xml_filename="0001_123456.xml",
            folder_element={"id": "folderid"},
            element_type="page",
            paragraph_type="paragraph",
            line_type="text_line",
        )
        test_element.upload_image()

    responses.assert_call_count("http://s3/put", 1)
    assert test_element.image, {
        "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
        "s3_put_url": None,
        "width": 999,
        "height": 999,
        "status": "checked",
    }


@patch("worker_file_import.transkribus.hash_image")
def test_upload_image_exists(hash_image_mock, mock_worker):
    hash_image_mock.return_value = "01e9327d0e5ab7c516a0792c3662bc65"

    mock_worker.api_client.add_error_response(
        "CreateImage",
        body={"hash": "01e9327d0e5ab7c516a0792c3662bc65"},
        status_code=400,
        content={"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"},
    )
    mock_worker.api_client.add_response(
        "RetrieveImage",
        id="aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
        response={
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "s3_url": "https://nerv.co.jp/images/something",
        },
    )
    mock_worker.api_client.add_response(
        "PartialUpdateImage",
        id="aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
        body={"status": "checked"},
        response={
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "s3_put_url": None,
            "width": 999,
            "height": 999,
            "status": "checked",
        },
    )

    with (
        NamedTemporaryFile() as archive_file,
        ZipFile(archive_file, mode="w") as archive,
    ):
        archive.write(str(SAMPLES / "200x200.jpg"), arcname="0001_123456.jpg")
        archive.write(str(SAMPLES / "transcript.xml"), arcname="0001_123456.xml")
        archive_file.flush()
        archive_file.seek(0)

        test_element = TranskribusElement(
            mock_worker,
            archive,
            image_filename="0001_123456.jpg",
            xml_filename="0001_123456.xml",
            folder_element={"id": "folderid"},
            element_type="page",
            paragraph_type="paragraph",
            line_type="text_line",
        )
        test_element.upload_image()

    assert test_element.image == {
        "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
        "s3_put_url": None,
        "width": 999,
        "height": 999,
        "status": "checked",
    }


@patch("worker_file_import.transkribus.hash_image")
def test_upload_image_failure(hash_image_mock, mock_worker):
    hash_image_mock.return_value = "01e9327d0e5ab7c516a0792c3662bc65"

    mock_worker.api_client.add_error_response(
        "CreateImage",
        body={"hash": "01e9327d0e5ab7c516a0792c3662bc65"},
        status_code=500,
        content={"oh no"},
    )

    with (
        NamedTemporaryFile() as archive_file,
        ZipFile(archive_file, mode="w") as archive,
    ):
        archive.write(str(SAMPLES / "200x200.jpg"), arcname="0001_123456.jpg")
        archive.write(str(SAMPLES / "transcript.xml"), arcname="0001_123456.xml")
        archive_file.flush()
        archive_file.seek(0)

        test_element = TranskribusElement(
            mock_worker,
            archive,
            image_filename="0001_123456.jpg",
            xml_filename="0001_123456.xml",
            folder_element={"id": "folderid"},
            element_type="page",
            paragraph_type="paragraph",
            line_type="text_line",
        )

        with pytest.raises(ErrorResponse):
            test_element.upload_image()


def test_upload_transcriptions(mock_worker):
    mock_worker.api_client.add_response(
        "CreateElement",
        body={
            "corpus": "corpusid",
            "parent": "folderid",
            "type": "page",
            "name": "42",
            "polygon": None,
            "confidence": None,
            "image": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "worker_run_id": mock_worker.worker_run_id,
        },
        response={
            "id": "elementid",
            "type": "page",
            "name": "200x200.jpg",
            "corpus": {"id": "corpusid"},
            "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
        },
    )
    mock_worker.api_client.add_response(
        "CreateMetaDataBulk",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "metadata_list": [
                {
                    "type": "reference",
                    "name": "transkribus_page_id",
                    "value": "1",
                    "entity_id": None,
                },
                {"type": "text", "name": "folio", "value": "42", "entity_id": None},
                {
                    "type": "text",
                    "name": "transkribus_status",
                    "value": "IN_PROGRESS",
                    "entity_id": None,
                },
            ],
        },
        id="elementid",
        response={"metadata_list": [{"id": "mdid1"}, {"id": "mdid2"}, {"id": "mdid3"}]},
    )
    mock_worker.api_client.add_response(
        "CreateMetaData",
        body={
            "type": "text",
            "name": "original_image_name",
            "value": "01R_CE101S01_1907_005.tif",
            "entity_id": None,
            "worker_run_id": mock_worker.worker_run_id,
        },
        id="elementid",
        response={"id": "mdid"},
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="elementid",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "type": "paragraph",
                    "name": "0",
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                },
                {
                    "type": "paragraph",
                    "name": "1",
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                },
            ],
        },
        response=[
            {
                "id": "region1",
                "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
            },
            {
                "id": "region2",
                "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
            },
        ],
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="region1",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "0",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "1",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "2",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "3",
                },
            ],
        },
        response=[{"id": "line1"}, {"id": "line2"}, {"id": "line3"}, {"id": "line4"}],
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="region2",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "0",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "1",
                },
            ],
        },
        response=[{"id": "line5"}, {"id": "line6"}],
    )

    for line_id, transkribus_id in [
        ("line1", "r1l6"),
        ("line2", "r1l7"),
        ("line3", "r1l8"),
        ("line4", "r1l9"),
        ("line5", "r2l12"),
        ("line6", "r2l13"),
    ]:
        mock_worker.api_client.add_response(
            "CreateMetaData",
            body={
                "type": "reference",
                "name": "transkribus_id",
                "value": transkribus_id,
                "worker_run_id": mock_worker.worker_run_id,
                "entity_id": None,
            },
            id=line_id,
            response={"id": "mdid"},
        )

    mock_worker.api_client.add_response(
        "CreateTranscriptions",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "transcriptions": [
                {
                    "element_id": "region1",
                    "text": "B .1\nLouis Joseph\nPierre Siméon\nLemieux",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line1",
                    "text": "B .1",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line2",
                    "text": "Louis Joseph",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line3",
                    "text": "Pierre Siméon",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line4",
                    "text": "Lemieux",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "region2",
                    "text": "Le onze janvier mil neuf centsept\nnous prêtre soussigné avons baptisé Louis",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line5",
                    "text": "Le onze janvier mil neuf centsept",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line6",
                    "text": "nous prêtre soussigné avons baptisé Louis",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
            ],
        },
        response={
            "transcriptions": [
                {"id": "transcription1"},
                {"id": "transcription2"},
                {"id": "transcription3"},
                {"id": "transcription4"},
                {"id": "transcription5"},
                {"id": "transcription6"},
                {"id": "transcription7"},
                {"id": "transcription8"},
            ]
        },
    )

    with (
        NamedTemporaryFile() as archive_file,
        ZipFile(archive_file, mode="w") as archive,
    ):
        archive.write(str(SAMPLES / "200x200.jpg"), arcname="0001_123456.jpg")
        archive.write(str(SAMPLES / "transcript.xml"), arcname="0001_123456.xml")
        archive_file.flush()
        archive_file.seek(0)

        test_element = TranskribusElement(
            mock_worker,
            archive,
            image_filename="0001_123456.jpg",
            xml_filename="0001_123456.xml",
            folder_element={"id": "folderid", "corpus": {"id": "corpusid"}},
            element_type="page",
            paragraph_type="paragraph",
            line_type="text_line",
        )
        test_element.page = {
            "id": "elementid",
            "corpus": {"id": "corpusid"},
            "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
        }
        test_element.image = {
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "width": 12558699,
            "height": 12558699,
        }
        test_element.upload_transcriptions()


@patch("worker_file_import.transkribus.hash_image")
def test_run(hash_image_mock, mock_worker, responses):
    hash_image_mock.return_value = "01e9327d0e5ab7c516a0792c3662bc65"

    mock_worker.api_client.add_response(
        "CreateImage",
        body={"hash": "01e9327d0e5ab7c516a0792c3662bc65"},
        response={
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "s3_put_url": "http://s3/put",
            "s3_url": "http://s3/image.jpg",
        },
    )
    responses.add(responses.PUT, "http://s3/put", status=201)
    mock_worker.api_client.add_response(
        "PartialUpdateImage",
        id="aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
        body={"status": "checked"},
        response={
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "s3_put_url": None,
            "width": 999,
            "height": 999,
            "status": "checked",
        },
    )
    mock_worker.api_client.add_response(
        "CreateElement",
        body={
            "corpus": "corpusid",
            "parent": "folderid",
            "type": "page",
            "name": "42",
            "polygon": None,
            "confidence": None,
            "image": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "worker_run_id": mock_worker.worker_run_id,
        },
        response={
            "id": "elementid",
            "type": "page",
            "name": "new elements",
            "corpus": {"id": "corpusid"},
            "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
        },
    )
    mock_worker.api_client.add_response(
        "CreateMetaDataBulk",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "metadata_list": [
                {
                    "type": "reference",
                    "name": "transkribus_page_id",
                    "value": "1",
                    "entity_id": None,
                },
                {"type": "text", "name": "folio", "value": "42", "entity_id": None},
                {
                    "type": "text",
                    "name": "transkribus_status",
                    "value": "IN_PROGRESS",
                    "entity_id": None,
                },
            ],
        },
        id="elementid",
        response={"metadata_list": [{"id": "mdid1"}, {"id": "mdid2"}, {"id": "mdid3"}]},
    )
    mock_worker.api_client.add_response(
        "CreateMetaData",
        body={
            "type": "text",
            "name": "original_image_name",
            "value": "01R_CE101S01_1907_005.tif",
            "entity_id": None,
            "worker_run_id": mock_worker.worker_run_id,
        },
        id="elementid",
        response={"id": "mdid"},
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="elementid",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "type": "paragraph",
                    "name": "0",
                    "polygon": [[12, 34], [56, 78], [910, 999], [12, 34]],
                },
                {
                    "type": "paragraph",
                    "name": "1",
                    "polygon": [[12, 34], [56, 78], [910, 999], [12, 34]],
                },
            ],
        },
        response=[
            {
                "id": "region1",
                "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
            },
            {
                "id": "region2",
                "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
            },
        ],
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="region1",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "polygon": [[12, 34], [56, 78], [910, 999], [12, 34]],
                    "type": "text_line",
                    "name": "0",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 999], [12, 34]],
                    "type": "text_line",
                    "name": "1",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 999], [12, 34]],
                    "type": "text_line",
                    "name": "2",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 999], [12, 34]],
                    "type": "text_line",
                    "name": "3",
                },
            ],
        },
        response=[{"id": "line1"}, {"id": "line2"}, {"id": "line3"}, {"id": "line4"}],
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="region2",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "polygon": [[12, 34], [56, 78], [910, 999], [12, 34]],
                    "type": "text_line",
                    "name": "0",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 999], [12, 34]],
                    "type": "text_line",
                    "name": "1",
                },
            ],
        },
        response=[{"id": "line5"}, {"id": "line6"}],
    )

    for line_id, transkribus_id in [
        ("line1", "r1l6"),
        ("line2", "r1l7"),
        ("line3", "r1l8"),
        ("line4", "r1l9"),
        ("line5", "r2l12"),
        ("line6", "r2l13"),
    ]:
        mock_worker.api_client.add_response(
            "CreateMetaData",
            body={
                "type": "reference",
                "name": "transkribus_id",
                "value": transkribus_id,
                "worker_run_id": mock_worker.worker_run_id,
                "entity_id": None,
            },
            id=line_id,
            response={"id": "mdid"},
        )

    mock_worker.api_client.add_response(
        "CreateTranscriptions",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "transcriptions": [
                {
                    "element_id": "region1",
                    "text": "B .1\nLouis Joseph\nPierre Siméon\nLemieux",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line1",
                    "text": "B .1",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line2",
                    "text": "Louis Joseph",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line3",
                    "text": "Pierre Siméon",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line4",
                    "text": "Lemieux",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "region2",
                    "text": "Le onze janvier mil neuf centsept\nnous prêtre soussigné avons baptisé Louis",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line5",
                    "text": "Le onze janvier mil neuf centsept",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line6",
                    "text": "nous prêtre soussigné avons baptisé Louis",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
            ],
        },
        response={
            "transcriptions": [
                {"id": "transcription1"},
                {"id": "transcription2"},
                {"id": "transcription3"},
                {"id": "transcription4"},
                {"id": "transcription5"},
                {"id": "transcription6"},
                {"id": "transcription7"},
                {"id": "transcription8"},
            ]
        },
    )

    with (
        NamedTemporaryFile() as archive_file,
        ZipFile(archive_file, mode="w") as archive,
    ):
        archive.write(str(SAMPLES / "200x200.jpg"), arcname="0001_123456.jpg")
        archive.write(str(SAMPLES / "transcript.xml"), arcname="0001_123456.xml")
        archive_file.flush()
        archive_file.seek(0)

        test_element = TranskribusElement(
            mock_worker,
            archive,
            image_filename="0001_123456.jpg",
            xml_filename="0001_123456.xml",
            folder_element={"id": "folderid", "corpus": {"id": "corpusid"}},
            element_type="page",
            paragraph_type="paragraph",
            line_type="text_line",
        )

        test_element.run()

    assert test_element.image == {
        "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
        "s3_put_url": None,
        "width": 999,
        "height": 999,
        "status": "checked",
    }
