from pathlib import Path
from tempfile import NamedTemporaryFile
from unittest.mock import patch
from zipfile import ZipFile

from arkindex_worker.models import Element
from worker_file_import.transkribus import TranskribusImporter

SAMPLES = Path(__file__).absolute().parent / "samples"
EXPECTED = SAMPLES / "expected"

CORPUS_INFO = {
    "id": "corpusid",
    "name": "Corpus",
    "types": [
        {"slug": "book", "display_name": "a book", "folder": True},
        {"slug": "page", "display_name": "a page", "folder": False},
        {"slug": "paragraph", "display_name": "a paragraph", "folder": False},
        {"slug": "text_line", "display_name": "a line", "folder": False},
    ],
}


# def setUpClass(cls):
# _, cls.archive_path = tempfile.mkstemp()
# cls.archive_path = Path(cls.archive_path)
# with ZipFile(cls.archive_path, mode="w") as archive:
# archive.write(
#     str(SAMPLES / "img1.jpg"), arcname="555/document/0001_123456.jpg"
# )
# archive.write(
#     str(SAMPLES / "transcript.xml"), arcname="555/document/0001_123456.xml"
# )

# _, cls.subdir_archive_path = tempfile.mkstemp()
# cls.subdir_archive_path = Path(cls.subdir_archive_path)
# with ZipFile(cls.subdir_archive_path, mode="w") as archive:
#     archive.write(
#         str(SAMPLES / "img1.jpg"),
#         arcname="555/document/some/subdir/abcdefg.jpg",
#     )
#     archive.write(
#         str(SAMPLES / "transcript.xml"),
#         arcname="555/document/foo/bar/abcdefg.xml",
#     )

# @classmethod
# def tearDownClass(cls):
# cls.archive_path.unlink()

# def setUp(self):
# self.maxDiff = None


def test_init_missing_types(mock_worker):
    mock_worker.process_information = {
        "id": "processid",
        "corpus": "corpusid",
        "element": None,
        "folder_type": "book",
        "element_type": "page",
    }
    mock_worker.api_client.add_response(
        "RetrieveCorpus",
        id="corpusid",
        response={
            "id": "corpusid",
            "name": "Corpus",
            "types": [
                {"slug": "book", "display_name": "a book", "folder": True},
                {"slug": "page", "display_name": "a page", "folder": False},
            ],
        },
    )
    mock_worker.api_client.add_response(
        "CreateElementType",
        body={
            "slug": "paragraph",
            "display_name": "Paragraph",
            "corpus": "corpusid",
        },
        response={"id": "par_type_id"},
    )
    mock_worker.api_client.add_response(
        "CreateElementType",
        body={
            "slug": "text_line",
            "display_name": "Text Line",
            "corpus": "corpusid",
        },
        response={"id": "line_type_id"},
    )

    with (
        NamedTemporaryFile() as archive_file,
        ZipFile(archive_file, mode="w") as archive,
    ):
        archive.write(
            str(SAMPLES / "200x200.jpg"), arcname="555/document/0001_123456.jpg"
        )
        archive.write(
            str(SAMPLES / "transcript.xml"), arcname="555/document/0001_123456.xml"
        )
        archive_file.flush()
        archive_file.seek(0)

        importer = TranskribusImporter(mock_worker, archive_file)

    assert importer.archive_path == archive_file
    assert importer.project_id == "corpusid"
    assert importer.folder_type == "book"
    assert importer.element_type == "page"
    assert importer.paragraph_type == "paragraph"
    assert importer.line_type == "text_line"
    assert importer.parent_element is None


@patch("worker_file_import.transkribus.hash_image")
def test_run(hash_image_mock, mock_worker, responses):
    hash_image_mock.return_value = "01e9327d0e5ab7c516a0792c3662bc65"
    mock_worker.process_information = {
        "id": "processid",
        "corpus": "corpusid",
        "element": None,
        "folder_type": "book",
        "element_type": "page",
    }

    mock_worker.api_client.add_response(
        "RetrieveCorpus", id="corpusid", response=CORPUS_INFO
    )
    mock_worker.api_client.add_response(
        "ListElements", corpus="corpusid", name="document", type="book", response=[]
    )
    mock_worker.api_client.add_response(
        "CreateImage",
        body={"hash": "01e9327d0e5ab7c516a0792c3662bc65"},
        response={
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "s3_put_url": "http://s3/put",
            "s3_url": "http://s3/image.jpg",
            "width": 0,
            "height": 0,
        },
    )
    responses.add(responses.PUT, "http://s3/put", status=201)
    mock_worker.api_client.add_response(
        "PartialUpdateImage",
        id="aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
        body={"status": "checked"},
        response={
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "s3_put_url": None,
            "width": 12558699,
            "height": 12558699,
            "status": "checked",
        },
    )
    mock_worker.api_client.add_response(
        "CreateElement",
        body={
            "corpus": "corpusid",
            "type": "book",
            "name": "document",
            "worker_run_id": mock_worker.worker_run_id,
        },
        response={
            "id": "folderid",
            "type": "book",
            "name": "document",
            "corpus": {"id": "corpusid"},
        },
    )
    mock_worker.api_client.add_response(
        "CreateElement",
        body={
            "corpus": "corpusid",
            "type": "page",
            "name": "42",
            "parent": "folderid",
            "image": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "worker_run_id": mock_worker.worker_run_id,
            "polygon": None,
            "confidence": None,
        },
        response={
            "id": "elementid",
            "type": "page",
            "name": "42",
            "parent": "folderid",
            "corpus": {"id": "corpusid"},
            "zone": {
                "image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"},
                "polygon": None,
            },
            "confidence": None,
        },
    )
    mock_worker.api_client.add_response(
        "CreateMetaDataBulk",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "metadata_list": [
                {
                    "type": "reference",
                    "name": "transkribus_page_id",
                    "value": "1",
                    "entity_id": None,
                },
                {"type": "text", "name": "folio", "value": "42", "entity_id": None},
                {
                    "type": "text",
                    "name": "transkribus_status",
                    "value": "IN_PROGRESS",
                    "entity_id": None,
                },
            ],
        },
        id="elementid",
        response={"metadata_list": [{"id": "mdid1"}, {"id": "mdid2"}, {"id": "mdid3"}]},
    )
    mock_worker.api_client.add_response(
        "CreateMetaData",
        body={
            "type": "text",
            "name": "original_image_name",
            "value": "01R_CE101S01_1907_005.tif",
            "entity_id": None,
            "worker_run_id": mock_worker.worker_run_id,
        },
        id="elementid",
        response={"id": "mdid"},
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="elementid",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "type": "paragraph",
                    "name": "0",
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                },
                {
                    "type": "paragraph",
                    "name": "1",
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                },
            ],
        },
        response=[
            {
                "id": "region1",
                "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
            },
            {
                "id": "region2",
                "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
            },
        ],
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="region1",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "0",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "1",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "2",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "3",
                },
            ],
        },
        response=[{"id": "line1"}, {"id": "line2"}, {"id": "line3"}, {"id": "line4"}],
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="region2",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "0",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "1",
                },
            ],
        },
        response=[{"id": "line5"}, {"id": "line6"}],
    )

    for line_id, transkribus_id in [
        ("line1", "r1l6"),
        ("line2", "r1l7"),
        ("line3", "r1l8"),
        ("line4", "r1l9"),
        ("line5", "r2l12"),
        ("line6", "r2l13"),
    ]:
        mock_worker.api_client.add_response(
            "CreateMetaData",
            body={
                "type": "reference",
                "name": "transkribus_id",
                "value": transkribus_id,
                "worker_run_id": mock_worker.worker_run_id,
                "entity_id": None,
            },
            id=line_id,
            response={"id": "mdid"},
        )

    mock_worker.api_client.add_response(
        "CreateTranscriptions",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "transcriptions": [
                {
                    "element_id": "region1",
                    "text": "B .1\nLouis Joseph\nPierre Siméon\nLemieux",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line1",
                    "text": "B .1",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line2",
                    "text": "Louis Joseph",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line3",
                    "text": "Pierre Siméon",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line4",
                    "text": "Lemieux",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "region2",
                    "text": "Le onze janvier mil neuf centsept\nnous prêtre soussigné avons baptisé Louis",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line5",
                    "text": "Le onze janvier mil neuf centsept",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line6",
                    "text": "nous prêtre soussigné avons baptisé Louis",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
            ],
        },
        response={
            "transcriptions": [
                {"id": "transcription1"},
                {"id": "transcription2"},
                {"id": "transcription3"},
                {"id": "transcription4"},
                {"id": "transcription5"},
                {"id": "transcription6"},
                {"id": "transcription7"},
                {"id": "transcription8"},
            ]
        },
    )

    with NamedTemporaryFile() as archive_file:
        with ZipFile(archive_file, mode="w") as archive:
            archive.write(
                str(SAMPLES / "200x200.jpg"), arcname="555/document/0001_123456.jpg"
            )
            archive.write(
                str(SAMPLES / "transcript.xml"), arcname="555/document/0001_123456.xml"
            )
            archive_file.flush()
            archive_file.seek(0)

        importer = TranskribusImporter(mock_worker, archive_file)

        assert importer.run() == [
            {
                "id": "folderid",
                "type": "book",
                "name": "document",
                "corpus": {"id": "corpusid"},
            },
            {
                "id": "elementid",
                "parent": "folderid",
                "type": "page",
                "name": "42",
                "corpus": {"id": "corpusid"},
                "zone": {
                    "image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"},
                    "polygon": None,
                },
                "confidence": None,
            },
        ]


@patch("worker_file_import.transkribus.hash_image")
def test_run_subdir(hash_image_mock, mock_worker, responses):
    hash_image_mock.return_value = "01e9327d0e5ab7c516a0792c3662bc65"
    mock_worker.process_information = {
        "id": "processid",
        "corpus": "corpusid",
        "element": None,
        "folder_type": "book",
        "element_type": "page",
    }

    mock_worker.api_client.add_response(
        "RetrieveCorpus", id="corpusid", response=CORPUS_INFO
    )
    mock_worker.api_client.add_response(
        "ListElements", corpus="corpusid", name="document", type="book", response=[]
    )
    mock_worker.api_client.add_response(
        "CreateImage",
        body={"hash": "01e9327d0e5ab7c516a0792c3662bc65"},
        response={
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "s3_put_url": "http://s3/put",
            "s3_url": "http://s3/image.jpg",
            "width": 0,
            "height": 0,
        },
    )
    responses.add(responses.PUT, "http://s3/put", status=201)
    mock_worker.api_client.add_response(
        "PartialUpdateImage",
        id="aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
        body={"status": "checked"},
        response={
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "s3_put_url": None,
            "width": 12558699,
            "height": 12558699,
            "status": "checked",
        },
    )
    mock_worker.api_client.add_response(
        "CreateElement",
        body={
            "corpus": "corpusid",
            "type": "book",
            "name": "document",
            "worker_run_id": mock_worker.worker_run_id,
        },
        response={
            "id": "folderid",
            "type": "book",
            "name": "document",
            "corpus": {"id": "corpusid"},
        },
    )
    mock_worker.api_client.add_response(
        "CreateElement",
        body={
            "corpus": "corpusid",
            "type": "page",
            "name": "42",
            "parent": "folderid",
            "image": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "worker_run_id": mock_worker.worker_run_id,
            "polygon": None,
            "confidence": None,
        },
        response={
            "id": "elementid",
            "type": "page",
            "name": "42",
            "parent": "folderid",
            "corpus": {"id": "corpusid"},
            "zone": {
                "image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"},
                "polygon": None,
            },
            "confidence": None,
        },
    )
    mock_worker.api_client.add_response(
        "CreateMetaDataBulk",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "metadata_list": [
                {
                    "type": "reference",
                    "name": "transkribus_page_id",
                    "value": "1",
                    "entity_id": None,
                },
                {"type": "text", "name": "folio", "value": "42", "entity_id": None},
                {
                    "type": "text",
                    "name": "transkribus_status",
                    "value": "IN_PROGRESS",
                    "entity_id": None,
                },
            ],
        },
        id="elementid",
        response={"metadata_list": [{"id": "mdid1"}, {"id": "mdid2"}, {"id": "mdid3"}]},
    )
    mock_worker.api_client.add_response(
        "CreateMetaData",
        body={
            "type": "text",
            "name": "original_image_name",
            "value": "01R_CE101S01_1907_005.tif",
            "entity_id": None,
            "worker_run_id": mock_worker.worker_run_id,
        },
        id="elementid",
        response={"id": "mdid"},
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="elementid",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "type": "paragraph",
                    "name": "0",
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                },
                {
                    "type": "paragraph",
                    "name": "1",
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                },
            ],
        },
        response=[
            {
                "id": "region1",
                "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
            },
            {
                "id": "region2",
                "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
            },
        ],
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="region1",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "0",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "1",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "2",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "3",
                },
            ],
        },
        response=[{"id": "line1"}, {"id": "line2"}, {"id": "line3"}, {"id": "line4"}],
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="region2",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "0",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "1",
                },
            ],
        },
        response=[{"id": "line5"}, {"id": "line6"}],
    )

    for line_id, transkribus_id in [
        ("line1", "r1l6"),
        ("line2", "r1l7"),
        ("line3", "r1l8"),
        ("line4", "r1l9"),
        ("line5", "r2l12"),
        ("line6", "r2l13"),
    ]:
        mock_worker.api_client.add_response(
            "CreateMetaData",
            body={
                "type": "reference",
                "name": "transkribus_id",
                "value": transkribus_id,
                "worker_run_id": mock_worker.worker_run_id,
                "entity_id": None,
            },
            id=line_id,
            response={"id": "mdid"},
        )

    mock_worker.api_client.add_response(
        "CreateTranscriptions",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "transcriptions": [
                {
                    "element_id": "region1",
                    "text": "B .1\nLouis Joseph\nPierre Siméon\nLemieux",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line1",
                    "text": "B .1",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line2",
                    "text": "Louis Joseph",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line3",
                    "text": "Pierre Siméon",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line4",
                    "text": "Lemieux",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "region2",
                    "text": "Le onze janvier mil neuf centsept\nnous prêtre soussigné avons baptisé Louis",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line5",
                    "text": "Le onze janvier mil neuf centsept",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line6",
                    "text": "nous prêtre soussigné avons baptisé Louis",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
            ],
        },
        response={
            "transcriptions": [
                {"id": "transcription1"},
                {"id": "transcription2"},
                {"id": "transcription3"},
                {"id": "transcription4"},
                {"id": "transcription5"},
                {"id": "transcription6"},
                {"id": "transcription7"},
                {"id": "transcription8"},
            ]
        },
    )

    with NamedTemporaryFile() as archive_file:
        with ZipFile(archive_file, mode="w") as archive:
            archive.write(
                str(SAMPLES / "200x200.jpg"),
                arcname="555/document/some/subdir/0001_123456.jpg",
            )
            archive.write(
                str(SAMPLES / "transcript.xml"),
                arcname="555/document/some/subdir/0001_123456.xml",
            )
            archive_file.flush()
            archive_file.seek(0)

        importer = TranskribusImporter(mock_worker, archive_file)

        assert importer.run() == [
            {
                "id": "folderid",
                "type": "book",
                "name": "document",
                "corpus": {"id": "corpusid"},
            },
            {
                "id": "elementid",
                "parent": "folderid",
                "type": "page",
                "name": "42",
                "corpus": {"id": "corpusid"},
                "zone": {
                    "image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"},
                    "polygon": None,
                },
                "confidence": None,
            },
        ]


@patch("worker_file_import.transkribus.hash_image")
def test_run_parent(hash_image_mock, mock_worker, responses):
    hash_image_mock.return_value = "01e9327d0e5ab7c516a0792c3662bc65"
    mock_worker.process_information = {
        "id": "processid",
        "corpus": "corpusid",
        "element": {"id": "parentid", "type": "book", "corpus": {"id": "corpusid"}},
        "folder_type": "book",
        "element_type": "page",
    }

    mock_worker.api_client.add_response(
        "RetrieveCorpus", id="corpusid", response=CORPUS_INFO
    )
    mock_worker.api_client.add_response(
        "ListElementChildren",
        id="parentid",
        name="document",
        type="book",
        recursive=True,
        response=[],
    )
    mock_worker.api_client.add_response(
        "CreateImage",
        body={"hash": "01e9327d0e5ab7c516a0792c3662bc65"},
        response={
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "s3_put_url": "http://s3/put",
            "s3_url": "http://s3/image.jpg",
            "width": 0,
            "height": 0,
        },
    )
    responses.add(responses.PUT, "http://s3/put", status=201)
    mock_worker.api_client.add_response(
        "PartialUpdateImage",
        id="aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
        body={"status": "checked"},
        response={
            "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "s3_put_url": None,
            "width": 12558699,
            "height": 12558699,
            "status": "checked",
        },
    )
    mock_worker.api_client.add_response(
        "CreateElement",
        body={
            "corpus": "corpusid",
            "type": "book",
            "name": "document",
            "parent": "parentid",
            "worker_run_id": mock_worker.worker_run_id,
            "image": None,
            "polygon": None,
            "confidence": None,
        },
        response={
            "id": "folderid",
            "type": "book",
            "name": "document",
            "parent": "parentid",
            "corpus": {"id": "corpusid"},
        },
    )
    mock_worker.api_client.add_response(
        "CreateElement",
        body={
            "corpus": "corpusid",
            "type": "page",
            "name": "42",
            "parent": "folderid",
            "image": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
            "worker_run_id": mock_worker.worker_run_id,
            "polygon": None,
            "confidence": None,
        },
        response={
            "id": "elementid",
            "type": "page",
            "name": "42",
            "parent": "folderid",
            "corpus": {"id": "corpusid"},
            "zone": {
                "image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"},
                "polygon": None,
            },
            "confidence": None,
        },
    )
    mock_worker.api_client.add_response(
        "CreateMetaDataBulk",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "metadata_list": [
                {
                    "type": "reference",
                    "name": "transkribus_page_id",
                    "value": "1",
                    "entity_id": None,
                },
                {"type": "text", "name": "folio", "value": "42", "entity_id": None},
                {
                    "type": "text",
                    "name": "transkribus_status",
                    "value": "IN_PROGRESS",
                    "entity_id": None,
                },
            ],
        },
        id="elementid",
        response={"metadata_list": [{"id": "mdid1"}, {"id": "mdid2"}, {"id": "mdid3"}]},
    )
    mock_worker.api_client.add_response(
        "CreateMetaData",
        body={
            "type": "text",
            "name": "original_image_name",
            "value": "01R_CE101S01_1907_005.tif",
            "entity_id": None,
            "worker_run_id": mock_worker.worker_run_id,
        },
        id="elementid",
        response={"id": "mdid"},
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="elementid",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "type": "paragraph",
                    "name": "0",
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                },
                {
                    "type": "paragraph",
                    "name": "1",
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                },
            ],
        },
        response=[
            {
                "id": "region1",
                "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
            },
            {
                "id": "region2",
                "zone": {"image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"}},
            },
        ],
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="region1",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "0",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "1",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "2",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "3",
                },
            ],
        },
        response=[{"id": "line1"}, {"id": "line2"}, {"id": "line3"}, {"id": "line4"}],
    )
    mock_worker.api_client.add_response(
        "CreateElements",
        id="region2",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "elements": [
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "0",
                },
                {
                    "polygon": [[12, 34], [56, 78], [910, 1112], [12, 34]],
                    "type": "text_line",
                    "name": "1",
                },
            ],
        },
        response=[{"id": "line5"}, {"id": "line6"}],
    )

    for line_id, transkribus_id in [
        ("line1", "r1l6"),
        ("line2", "r1l7"),
        ("line3", "r1l8"),
        ("line4", "r1l9"),
        ("line5", "r2l12"),
        ("line6", "r2l13"),
    ]:
        mock_worker.api_client.add_response(
            "CreateMetaData",
            body={
                "type": "reference",
                "name": "transkribus_id",
                "value": transkribus_id,
                "worker_run_id": mock_worker.worker_run_id,
                "entity_id": None,
            },
            id=line_id,
            response={"id": "mdid"},
        )

    mock_worker.api_client.add_response(
        "CreateTranscriptions",
        body={
            "worker_run_id": mock_worker.worker_run_id,
            "transcriptions": [
                {
                    "element_id": "region1",
                    "text": "B .1\nLouis Joseph\nPierre Siméon\nLemieux",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line1",
                    "text": "B .1",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line2",
                    "text": "Louis Joseph",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line3",
                    "text": "Pierre Siméon",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line4",
                    "text": "Lemieux",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "region2",
                    "text": "Le onze janvier mil neuf centsept\nnous prêtre soussigné avons baptisé Louis",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line5",
                    "text": "Le onze janvier mil neuf centsept",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
                {
                    "element_id": "line6",
                    "text": "nous prêtre soussigné avons baptisé Louis",
                    "confidence": 1.0,
                    "orientation": "horizontal-lr",
                },
            ],
        },
        response={
            "transcriptions": [
                {"id": "transcription1"},
                {"id": "transcription2"},
                {"id": "transcription3"},
                {"id": "transcription4"},
                {"id": "transcription5"},
                {"id": "transcription6"},
                {"id": "transcription7"},
                {"id": "transcription8"},
            ]
        },
    )

    with NamedTemporaryFile() as archive_file:
        with ZipFile(archive_file, mode="w") as archive:
            archive.write(
                str(SAMPLES / "200x200.jpg"),
                arcname="555/document/some/subdir/0001_123456.jpg",
            )
            archive.write(
                str(SAMPLES / "transcript.xml"),
                arcname="555/document/some/subdir/0001_123456.xml",
            )
            archive_file.flush()
            archive_file.seek(0)

        importer = TranskribusImporter(
            mock_worker,
            archive_file,
            parent_element=Element(
                {"id": "parentid", "type": "book", "corpus": {"id": "corpusid"}}
            ),
        )

        assert importer.run() == [
            {
                "id": "folderid",
                "type": "book",
                "name": "document",
                "parent": "parentid",
                "corpus": {"id": "corpusid"},
            },
            {
                "id": "elementid",
                "parent": "folderid",
                "type": "page",
                "name": "42",
                "corpus": {"id": "corpusid"},
                "zone": {
                    "image": {"id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"},
                    "polygon": None,
                },
                "confidence": None,
            },
        ]


@patch("worker_file_import.transkribus.TranskribusFolder.run")
def test_run_folder_error(run_mock, mock_worker):
    mock_worker.process_information = {
        "id": "processid",
        "corpus": "corpusid",
        "element": {"id": "parentid", "type": "book", "corpus": {"id": "corpusid"}},
        "folder_type": "book",
        "element_type": "page",
    }

    mock_worker.api_client.add_response(
        "RetrieveCorpus", id="corpusid", response=CORPUS_INFO
    )
    run_mock.side_effect = TypeError("Oh snap!")

    with NamedTemporaryFile() as archive_file:
        with ZipFile(archive_file, mode="w") as archive:
            archive.write(
                str(SAMPLES / "200x200.jpg"),
                arcname="555/document/some/subdir/0001_123456.jpg",
            )
            archive.write(
                str(SAMPLES / "transcript.xml"),
                arcname="555/document/some/subdir/0001_123456.xml",
            )
            archive_file.flush()
            archive_file.seek(0)

        importer = TranskribusImporter(
            mock_worker,
            archive_file,
        )

        # Nothing is imported
        assert importer.run() == []
        assert run_mock.call_count == 1
