import hashlib
import json
import shutil
import tarfile
import tempfile
from logging import Logger, getLogger
from pathlib import Path
from tempfile import NamedTemporaryFile, TemporaryFile
from zipfile import ZipFile

import magic
import requests
from apistar.exceptions import ErrorResponse
from teklia_toolbox.requests import (
    HTTP_GET_RETRY_BACKOFF,
    download_file,
    should_verify_cert,
)
from tenacity import retry, retry_if_exception_type, stop_after_attempt, wait_fixed
from zstandard import ZstdDecompressor

logger: Logger = getLogger(__name__)

# application/zip is the standard, but all browsers on Windows
# upload files with application/x-zip-compressed instead.
ZIP_MIME_TYPES = {
    "application/zip",
    "application/x-zip-compressed",
}

TAR_MIME_TYPES = {
    # .tar
    "application/x-tar",
    # .tar.gz, but browsers will usually only detect the .gz
    "application/x-gtar",
    # .gz
    "application/gzip",
    # .bz2
    "application/x-bzip",
    "application/x-bzip2",
    # .lzma
    "application/x-lzma",
    # .xz
    "application/x-xz",
}

ZSTANDARD_MIME_TYPES = {
    "application/zstd",
}


def hash_image(path) -> str:
    md5 = hashlib.md5()
    with Path(path).open("rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            md5.update(chunk)
    return md5.hexdigest()


def download_files(api_client, datafiles):
    temp_path = tempfile.mkdtemp()
    files = []
    for datafile_id in datafiles:
        df = api_client.request("RetrieveDataFile", id=datafile_id)
        path = Path(temp_path) / datafile_id

        logger.info("Downloading file {} ({})".format(df["name"], df["id"]))
        try:
            download_file(df["s3_url"], path)
        except requests.RequestException as e:
            logger.error(
                "Failed downloading file {} ({}): {}".format(
                    df["name"], df["id"], str(e)
                )
            )
            continue

        df["local_path"] = path
        files.append(df)

    return files


def dump_json(elements, dir, path=None, filename="elements.json"):
    file_path = path and Path(path, filename)
    if not file_path:
        file_path = dir / filename
    assert len(elements), f"No elements could be written to {file_path}"

    with file_path.open("w") as f:
        json.dump(elements, f, indent=4)


def delete_files(api_client, files):
    """
    Delete DataFiles once they are imported.
    Deletion errors will only show warnings instead of making the import fail.
    """
    for datafile in files:
        try:
            api_client.request("DestroyDataFile", id=datafile["id"])
        except ErrorResponse as e:
            logger.warning(f'Could not delete file {datafile["name"]}: {e}')


@retry(
    reraise=True,
    retry=retry_if_exception_type(requests.RequestException),
    stop=stop_after_attempt(2),
    wait=wait_fixed(HTTP_GET_RETRY_BACKOFF),
)
def retried_get(url, headers, timeout, **kwargs):
    """
    Wrapper for requests.get with a retry mechanism.
    The same GET call will be retried 2 times.
    If the 2nd call still gives an exception, that exception is re-raised
    and the caller should catch it
    """
    with requests.get(
        url, headers=headers, timeout=timeout, verify=should_verify_cert(url), **kwargs
    ) as r:
        r.raise_for_status()
        return r


def get_archive_datafiles(datafile):
    """
    Generator that returns DataFile dicts for each file within an archive DataFile.
    The `local_path` is only guaranteed to exist until the next `next()` call.
    """
    if datafile["content_type"] in ZIP_MIME_TYPES:
        with ZipFile(datafile["local_path"]) as archive:
            for fileinfo in archive.infolist():
                with (
                    archive.open(fileinfo) as source,
                    NamedTemporaryFile() as extracted_file,
                ):
                    shutil.copyfileobj(source, extracted_file)
                    # Very small files may not be fully written to disk, which could cause
                    # the MIME type detection to find an empty file
                    extracted_file.flush()

                    yield {
                        "id": datafile["id"],
                        "name": fileinfo.filename,
                        "size": fileinfo.file_size,
                        "status": "checked",
                        # The file import relies on MIME types assigned to DataFiles,
                        # but we do not have them from within archives, so guess them with libmagic
                        "content_type": magic.from_file(extracted_file.name, mime=True),
                        "local_path": extracted_file.name,
                    }

    elif datafile["content_type"] in TAR_MIME_TYPES | ZSTANDARD_MIME_TYPES:
        if datafile["content_type"] in ZSTANDARD_MIME_TYPES:
            # TarFile does not support .tar.zst, so we decompress ourselves
            fileobj = TemporaryFile()
            with Path(datafile["local_path"]).open("rb") as compressed_stream:
                ZstdDecompressor().copy_stream(compressed_stream, fileobj)
            fileobj.seek(0)
        else:
            # Just open the file and let Tarfile deal with other compression methods
            with Path(datafile["local_path"]) as compressed_stream:
                fileobj = compressed_stream.open("rb")

        # We have to include `with fileobj` because TarFile() will not close the original file object by itself
        with fileobj, tarfile.open(fileobj=fileobj) as archive:
            # Block all of the tar features that could cause security issues
            # See <https://docs.python.org/3/library/tarfile.html#tarfile-extraction-filter>
            archive.extraction_filter = tarfile.data_filter

            for fileinfo in archive.getmembers():
                # Skip directories, symbolic and hard links, character/block devices and FIFOs
                if not fileinfo.isfile():
                    continue

                with (
                    archive.extractfile(fileinfo) as source,
                    NamedTemporaryFile() as extracted_file,
                ):
                    shutil.copyfileobj(source, extracted_file)

                    yield {
                        "id": datafile["id"],
                        "name": fileinfo.name,
                        "size": fileinfo.size,
                        "status": "checked",
                        # The file import relies on MIME types assigned to DataFiles,
                        # but we do not have them from within archives, so guess them with libmagic
                        "content_type": magic.from_file(extracted_file.name, mime=True),
                        "local_path": extracted_file.name,
                    }

    else:
        raise NotImplementedError
